﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AllocationTest
{
	class Program
	{
		static void Main(string[] args)
		{
			new Program().Run(args);
		}


		private void Run(string[] args)
		{
			if (args.Length == 0)
			{
				Help();
				Environment.Exit(1);
			}
			else if (args.Length == 1)
			{
				Console.WriteLine("Size\tCount\tTime\tGen0\tGen1\tGen2\tAllocGen");
				Environment.Exit(0);
			}

			int baseAlloc, allocSize, allocCount;

			if (!Int32.TryParse(args[0], out baseAlloc))
			{
				Console.WriteLine(string.Format("Cannot parse {0} to an integer", args[0]));
				Environment.Exit(1);
			}

			if (!Int32.TryParse(args[1], out allocSize))
			{
				Console.WriteLine(string.Format("Cannot parse {0} to an integer", args[1]));
				Environment.Exit(1);
			}

			if (!Int32.TryParse(args[2], out allocCount))
			{
				Console.WriteLine(string.Format("Cannot parse {0} to an integer", args[2]));
				Environment.Exit(1);
			}
			System.Diagnostics.Stopwatch sw = new System.Diagnostics.Stopwatch();
			sw.Start();
			
			var o = new byte[allocSize];
			int genTest = GC.GetGeneration(o);

			RunTest(baseAlloc, allocSize, allocCount);
			sw.Stop();
			Console.Write(allocSize +"\t" + allocCount + "\t");
			Console.WriteLine(sw.ElapsedMilliseconds + "\t" + GC.CollectionCount(0) + "\t" + GC.CollectionCount(1) + "\t" + GC.CollectionCount(2) + "\t" + genTest);
		}

		private readonly int MemoryPressureObjectSize = IntPtr.Size * 3;
		private class MemoryPressureObject
		{
			public int Level;
			public MemoryPressureObject A;
			public MemoryPressureObject B;
		}

		private static MemoryPressureObject _root;
		private void RunTest(int baseAlloc, int allocSize, int allocCount)
		{
			_root = new MemoryPressureObject() { Level = 0 };

			int objectsToAllocate = baseAlloc / MemoryPressureObjectSize;
			int requiredLevels = (int)Math.Log(objectsToAllocate, 2);
			int objectsAllocated = 0;

			Queue<MemoryPressureObject> l = new Queue<MemoryPressureObject>();
			l.Enqueue(_root);
			while (objectsAllocated < objectsToAllocate)
			{
				var m = l.Dequeue();

				m.A = new MemoryPressureObject() { Level = m.Level + 1 } ;
				//if (m.A.Level < requiredLevels)
				{
					l.Enqueue(m.A);
				}
				objectsAllocated++;

				m.B = new MemoryPressureObject() { Level = m.Level + 1 };
				if (m.B.Level < requiredLevels && l.Count < 8000)
				{
					l.Enqueue(m.B);
				}
				objectsAllocated++;
			}


			for (int i = 0; i < allocCount - 1; i++)
			{
				var o = new byte[allocSize];
			}


		}

		private void Help()
		{
			Console.WriteLine("AllocationTest.exe baseAlloc allocSize allocNum");
		}
	}
}
